function [iota,rho,lambda,V,xmax,prof,ms,prices,prodsur,conssur,expfreq, ...
    tildes,cond_num] = compOE(outfile,constants,transition,profpar,control,initial)
% INPUTS:   - output file, constants,transition,profpar, control, and initial parameters
%           
% OUTPUTS:  - OE investment strategy iota and exit strategy rho (xmax x 1)
%           - OE entry rate lambda
%           - OE value function V (xmax x 1)
%           - xmax: max acheivable state determined endogenously
%           - OE profits: prof,ms,prices,prodsur,conssur (xmax x 1)
%           - expfreq (xmax x 1): expected number of visits
%             to each state by one firm under OE strats.
%           - tildes(xmax x 1): OE expected state
%           - cond_num: condition number matrix inversion for expfreq
%
% Computes OE in entry/exit case

function [iota,rho,lambda,V,xmax,prof,ms,prices,prodsur,conssur,invdistr, ...
    tildes,cond_num] = compOE_fixn(outfile,constants,transition,profpar,control,initial)
% INPUTS:   - output file,constants,transition,profpar, control, and initial parameters
%           
% OUTPUTS:  - OE investment strategy iota and exit strategy rho (xmax x 1)
%           - lambda: number of firms 
%           - OE value function V (xmax x 1)
%           - xmax: max acheivable state determined endogenously
%           - OE profits: prof,ms,prices,prodsur,conssur (xmax x 1)
%           - invdistr (xmax x 1): invariant distr. of Markov chain that 
%             describes evolution of one firm under OE strats   
%           - tildes(xmax x 1): OE expected state
%           - cond_num: condition number matrix inversion for invdistr
%
% Computes OE in no entry/no exit case

function [iotaout,rhoout,xmaxout,Vout,prof,ms,prices,prodsur,conssur,expfreq, ...
    tildes, cond_num] = fpstratxmax(constants,transition,profpar, ...
    control,lambda,iota,rho,xmax,ignore)
% INPUTS:   - constants,transition,profpar, and control parameters
%           - entry rate lambda 
%           - initial investment strategy iota (xmax x 1)
%           - initial exit strategy rho (xmax x 1)
%           - initial value xmax 
%           - variable ignore: ignore=1 if the presence of "my" firm is 
%             ignored when computing profits. In this way only 
%             need to compute profits once 
% OUTPUTS:  - investment strategy iotaout and exit strategy rhoout (xmax x 1)
%             which is fixed point given entry rate lambda
%           - xmaxout: max acheivable state determined endogenously
%           - value function Vout associated to fixed point (xmax x 1)
%           - profits assocciated to fixed point:
%             prof,ms,prices,prodsur,conssur (xmax x 1)
%           - expfreq (xmax x 1): if entry and exit, expected number of visits
%             to each state by one firm under fixed point strats.
%             if no entry/no exit, invariant distr.
%             of Markov chain that describes evolution of one firm    
%           - tildes: expected state under fixed point strats. (xmax x 1)
%           - cond_num: condition number matrix inversion for expfreq

function [iotaout,rhoout,Vout,prof,ms,prices,prodsur,conssur,expfreq, ...
    tildes, cond_num] = fpstrat(constants,transition,profpar, ...
    control,lambda,iota1,rho1,xmax,ignore)
% INPUTS:   - constants,transition,profpar, and control parameters
%           - entry rate lambda 
%           - initial investment strategy iota1 (xmax x 1)
%           - initial exit strategy rho1 (xmax x 1)
%           - initial value xmax 
%           - variable ignore: ignore=1 if the presence of "my" firm is 
%             ignored when computing profits. In this way only 
%             need to compute profits once 
% OUTPUTS:  - investment strategy iotaout and exit strategy rhoout (xmax x 1)
%             which is fixed point given entry rate lambda
%           - value function Vout associated to fixed point (xmax x 1)
%           - profits assocciated to fixed point:
%             prof,ms,prices,prodsur,conssur (xmax x 1)
%           - expfreq (xmax x 1): if entry and exit, expected number of visits
%             to each state by one firm under fixed point strats.
%             if no entry/no exit, invariant distr.
%             of Markov chain that describes evolution of one firm    
%           - tildes: expected state under fixed point strats. (xmax x 1)
%           - cond_num: condition number matrix inversion for expfreq

function [expfreq, tildes, cond_num] = expstate(constants,control,B,lambda,xmax)

% INPUTS:   - constants and control parameters
%           - matrix of continutaion and transition probs. B (xmax x xmax)
%           - entry rate lambda
%           - xmax 
% OUPUTS:   - if fix_n=0 (entry/exit) expfreq (xmax x 1) is expected 
%             number of visits to each state by one firm 
%             if fix_n ~=0 (no entry/no exit) expfreq is invariant distr.
%             of Markov chain that describes evolution of one firm
%           - expected state tildes associated to B and lambda (xmax x 1)
%           - condition number of matrix (I-B) cond_num

function [iota,rho,Vnew,tranmat,prcont] = valueit(constants,transition,control,prof,xmax)
%
% INPUTS:   - constants, transition, and control parameters
%           - profit function prof (xmax x 1)
%           - xmax
% OUTPUTS:  - optimal strategies iota, rho given profits prof (xmax x 1)
%           - optimal value function Vnew given profits prof  (xmax x 1)
%           - transition probs tranmat given iota (xmax x xmax)
%           - continuation probs prcont given rho (xmax x 1)
% Solves Bellman's equation using value iteration

function iota = optinv(constants,transition,Vold,xmax)
% INPUTS:   - constants, transition parameters
%           - value function Vold (xmax x 1)
%           - xmax
% OUTPUTS:  - optimal strategies iota, given Vold (xmax x 1)
%
% Assumes transition dynamics in Weintraub, Benkard, and Van Roy (2005)
% to compute optimal investment

function [tranmat] = tranprob(transition,iota,xmax)

% INPUTS:   - transition parameters 
%           - investment strategy iota (xmax x 1) 
%           - xmax
% OUPUTS:   - matrix of transition probabilities tranmat (xmax x xmax)
%           
% Assumes transition dynamics in Weintraub, Benkard, and Van Roy (2005)

function [prcont,condexp] = contprob(transition,rho)

% INPUTS:   - transition parameters 
%           - exit strategy rho (cut-off values) (xmax x 1)
% OUPUTS:   - prcont: prob of continuation at every state (xmax x 1)
%           - conditional expectation of sell-off value
%             conditional on exiting (xmax x 1)
%           
% Sell-off values are i.i.d. exponential r.v.'s with mean K.

function [conttranmat] = conttranprob(prcont,tranmat,xmax)

% INPUTS:   - prcont: prob of continuation at every state (xmax x 1)
%           - matrix of transition probabilities tranmat (xmax x xmax)
%           - xmax
% OUPUTS:   - contranmat(x,y)= prob. of continuing in the industry in state
%             x and transit to state y (xmax x xmax)

function [prof,ms,prices,prodsur,conssur] = comp_prof(profpar,state,xmax,ignore)

% INPUTS:   - profit parameters profpar
%           - state (xmax x 1)
%           - xmax 
%           - variable ignore: ignore=1 if the presence of "my" firm is 
%             ignored when computing profits. In this way only 
%             need to compute profits once 
% OUTPUTS:  - profit function prof (xmax x 1)
%           - market shares ms (xmax x 1)
%           - prices (xmax x 1)
%           - producer surplus prodsur (xmax x 1)
%           - consumer surplus conssur (xmax x 1)
% 
% Calls function profit

function [prof,ms,prices,prodsur,conssur] = profit(profpar,state,p0)

% INPUTS:   - profit parameters profpar
%           - state (xmax x 1)
%           - initial condition for pricing game p0 (xmax x 1)
% OUTPUTS:  - profit function prof (xmax x 1)
%           - market shares ms (xmax x 1)
%           - prices (xmax x 1)
%           - producer surplus prodsur (scalar)
%           - consumer surplus conssur (scalar)
% 
% Profit function in Weintraub, Benkard, and Van Roy (2005)
% price competition with product differentiation
% logit demand system
% constant marginal cost of production
% Calls function focprice which are first order conditions of pricing game
% The i'th component of vectors prof,ms, and prices, 
% are the profit, market share, and price, respectively, 
% that a firm in state i would receive (if there was one in that state).

function x = focprice(p,profpar,state,w)

% INPUTS:   - price p (xmax x 1)
%           - profit parameters profpar    
%           - state (xmax x 1)
%           - utility function quality scaling w (xmax x 1)
% OUTPUT:   - FOCs for pricing game (logit demand system) (xmax x 1)

function Jnew = optstop(constants,transition,control)

% INPUTS:   - constant, transition, and control parameters 
% OUPUTS:   - barphi: expected NPV of entering the market, investing 0 each
%             period, and then exiting at an optimal stopping time

function [bound1,bound2,bound3,expV,entryrate,exitrate,msexiters,producersur,consumersur,totalsur...
    C1,C4,C10,C20] = compbounds(constants,transition,profpar,control,tildes,lambda,iota,rho,xmax,ignore)

% INPUTS:   - constants,transition,profpar, and control parameters
%           - OE expected state tildes (xmax x 1)
%           - OE entry rate lambda if entry/exit case
%             if no entry/no exit, lambda is number of firms    
%           - OE strategies iota, rho (xmax x 1)
%           - OE xmax
%           - variable ignore for computing profits
% OUTPUTS:  - bound1: estimator, estimator in percentage terms (normalized by expected
%             value function), and relative precision for loosest bound
%            (bound1 is valid for all starting states).
%           - bound2: estimator, estimator in percentage terms (normalized by expected
%             value function), and relative precision for tighter bound
%            (bound2 is a vector: it depends on the starting state).
%           - bound3: estimator, estimator in percentage terms (normalized by expected
%             value function), and relative precision for even tighter bound
%             that assumes firm can only change its state in one at every period
%            (bound3 is a vector: it depends on the starting state).
%           - expV: estimator and relative precision for value function (expectation
%             taken wrt invariant distr. of competitors state)
%           - Expected value (taken wrt to inavariant distr. of industry state) and stdev of:
%                   - entryrate (number of entrants/total number of firms)
%                   - exitrate  (number of exiters/total number of firms)
%                   - msexiters (market share firms that exit)
%                   - producer, consumer, and total surplus
%                   - concentration ratios C1, C4, C10, and C20

function  [xavg,xpre,xvar]= compavgpre(xsum,xsqr,n)
% INPUT:    - sum of X_i's xsum
%           - sum of sqrs of X_i's xsqr
%           - n number of obs.
% OUTPUT:   - xavg=mean(X_i)
%           - xvar=Var(X_i)
%           - xpre = relative precision of estimate with 98% confidence
%             level

function s=sample_poisson(tildes,xmax)

% INPUTS:   - state tildes (xmax x 1)
%           - xmax
% OUTPUT:   - random vector s distributed Poission with mean tildes 
%             (xmax x 1)

function [s,num_entr]=sample_constentr(lambda,Prob,p,xmax)

% INPUTS:   - entry rate lambda
%           - matrix Prob: row i-th (xmax x 1) is a vector with probs. of 
%             being in each state after i periods in the industry
%           - p: prob. to randomize number of entrants if lambda
%             non-integer
%           - xmax 
% OUTPUT:   - random vector s distributed according to invariant distr.
%             of competitors' state when fixed number of entrants (xmax x 1)
%           - num_entr: number of entrants

function s=sample_fix_n(sta_distr,xmax,num_firms)

% INPUTS:   - invariant distribution of evolution of one firm sta_distr
%             (xmax x 1)
%           - xmax
%           - number of competitors num_firms
% OUTPUT:   - random vector s sampled from invariant distr. of competitors'
%             state (xmax x 1)
