function [bound1,bound2,bound4,bound5,expV,entryrate,exitrate,msexiters,producersur,consumersur,totalsur...
    C1,C4,C10,C20] = compbounds(constants,transition,profpar,control,tildes,lambda,iota,rho,xmax,ignore)

% INPUTS:   - constants,transition,profpar, and control parameters
%           - OE expected state tildes (xmax x 1)
%           - OE entry rate lambda if entry/exit case
%             if no entry/no exit, lambda is number of firms    
%           - OE strategies iota, rho (xmax x 1)
%           - OE xmax
%           - variable ignore for computing profits
% OUTPUTS:  - bound1: estimator, estimator in percentage terms (normalized by expected
%             value function), and relative precision for loosest bound
%            (bound1 is valid for all starting states).
%           - bound2: estimator, estimator in percentage terms (normalized by expected
%             value function), and relative precision for tighter bound
%            (bound2 is a vector: it depends on the starting state).
%           - bound4: estimator, estimator in percentage terms (normalized by expected
%             value function), and relative precision for  tighter bound
%             that assumes firm can only change its state in one at every period
%            (bound4 is a vector: it depends on the starting state).
%           - expV: estimator and relative precision for value function (expectation
%             taken wrt invariant distr. of competitors state)
%           - Expected value (taken wrt to inavariant distr. of industry state) and stdev of:
%                   - entryrate (number of entrants/total number of firms)
%                   - exitrate  (number of exiters/total number of firms)
%                   - msexiters (market share firms that exit)
%                   - producer, consumer, and total surplus
%                   - concentration ratios C1, C4, C10, and C20
%

% Rename parameters to be used in this function
% to simplify notation
beta=constants.beta;
d=constants.d;
xe=constants.xe;
fix_n=constants.fix_n;
entryproc=constants.entryproc;

delta=transition.delta;

kmax_bound=control.kmax_bound;
TOL_precbounds=control.TOL_precbounds;
TOL_precV=control.TOL_precV;
NMAX_bound=control.NMAX_bound;
NMIN_bound=control.NMIN_bound;
ini_x_bound=control.ini_x_bound;
dis_x_bound=control.dis_x_bound;
defmaxbound=control.defmaxbound;
dis_opt_bound=control.dis_opt_bound;
TOL_time_cons_entr=control.TOL_time_cons_entr;
TOL_sample_cons_entr=control.TOL_sample_cons_entr;

TOL_precbounds=TOL_precbounds/(1+TOL_precbounds);
TOL_precV=TOL_precV/(1+TOL_precV);


xmax_bound=xmax*kmax_bound;   % Update xmax and tildes to compute bounds
tildes_ext=[tildes;zeros(xmax_bound-xmax,1)];

% Initialize variables
bound1sum=0;                bound1sqr=0;
bound2sum=zeros(xmax,1);    bound2sqr=zeros(xmax,1);
%bound3sum=zeros(size(ini_x_bound:dis_x_bound:xmax-dis_x_bound,2),...
%    size(ini_x_bound+dis_opt_bound:dis_opt_bound:xmax_bound-dis_opt_bound,2)+1);           
%bound3sqr=zeros(size(ini_x_bound:dis_x_bound:xmax-dis_x_bound,2),...
%    size(ini_x_bound+dis_opt_bound:dis_opt_bound:xmax_bound-dis_opt_bound,2)+1);            
bound4sum=zeros(size(ini_x_bound:dis_x_bound:xmax-dis_x_bound,2), 1);    
bound4sqr=zeros(size(ini_x_bound:dis_x_bound:xmax-dis_x_bound,2), 1);
bound5sum=zeros(size(ini_x_bound:dis_x_bound:xmax-dis_x_bound,2), 1);    
bound5sqr=zeros(size(ini_x_bound:dis_x_bound:xmax-dis_x_bound,2), 1);
boundspre=10*TOL_precbounds;    Vpre=10*TOL_precV;
Vsum=zeros(xmax,1);         Vsqr=zeros(xmax,1);
entratesum=0;               entratesqr=0;
exiratesum=0;               exiratesqr=0;
msexitsum=0;                msexitsqr=0;       
prodsursum=0;               prodsursqr=0;
conssursum=0;               conssursqr=0;
totalsursum=0;              totalsursqr=0;
c1sum=0;                    c1sqr=0;
c4sum=0;                    c4sqr=0;
c10sum=0;                   c10sqr=0;
c20sum=0;                   c20sqr=0;

%difprofsum=zeros(xmax_bound,1);

[proftildes,mstildes,pricestildes,prodsurtildes,conssurtildes]...
    = comp_prof(profpar,tildes_ext,xmax_bound,ignore);

% To compute expV, we compute "deterministic part" of value function 
% which is independent of the competitors' state under OE strats.
[prcont,condexp] = contprob(transition,rho);
tranmat = tranprob(transition,iota,xmax);
conttranmat = conttranprob(prcont,tranmat,xmax);
disc_Freq =(eye(xmax)-beta*conttranmat)\eye(xmax);
disc_Vdet = disc_Freq*((1-prcont).*condexp + (prcont).*(-d*iota));
bound2det=disc_Freq*proftildes(1:xmax);

% Compute prob. distributions to sample state if fixed number of entrants
if(entryproc==2 && fix_n==0)
    stop=0;
    time=1;
    A=eye(xmax);
    while(stop == 0 && time<TOL_time_cons_entr)
       Prob(time,:)=A(xe,:);     % row time of matrix Prob is a vector with probs of being
                                 % in each state after time periods in the industry
       stop=(sum(Prob(time,:)) < TOL_sample_cons_entr);      
       A=A*conttranmat;
       time=time+1;
    end
    % Compute prob. to randomize number of entrants if lambda non-integer
    if(ceil(lambda)~=floor(lambda))
        p=(ceil(lambda)-lambda)/(ceil(lambda)-floor(lambda));    
    else
        p=0;
    end    
    if(time==TOL_time_cons_entr)
        warning('Time limit reached when computing distribution to sample states with fixed number of entrants');
    end
end

% If fixed number of firms, compute invariant distribution
if(fix_n~=0)
    sta_distr=tildes/(fix_n-1);
end

    
    
state=zeros(xmax_bound,1);
n=1;
while((max(boundspre>TOL_precbounds) || max(Vpre>TOL_precV) || ...
        n<=NMIN_bound) && n<=NMAX_bound) 
    
    % sample state depending on entry process
    if(fix_n~=0)
        state(1:xmax)=sample_fix_n(sta_distr,xmax,fix_n-1);
    elseif(entryproc==1)
        state(1:xmax)=sample_poisson(tildes,xmax);
        entrants=sample_poisson(lambda,1);
    elseif(entryproc==2)
        [state(1:xmax),entrants]=sample_constentr(lambda,Prob,p,xmax);
    end
        
    [profstate,msstate,pricesstate,prodsurstate,conssurstate]...
        = comp_prof(profpar,state,xmax_bound,ignore);
    
    max_absd=max(abs(profstate-proftildes))*(2/(1-beta));
    bound1sum=bound1sum+max_absd;
    bound1sqr=bound1sqr+max_absd^2;
    
    discprof=disc_Freq*profstate(1:xmax);
    
    difprof=profstate-proftildes;
    difprofpos=max(difprof,0);
    
    if(fix_n~=0)
        max_d = max(difprof)/(1-beta);
    else
        max_d = max(max(difprof)/(1-beta),0);
    end
  
    bound2sum = bound2sum + (max_d + bound2det - discprof);
    bound2sqr = bound2sqr + (max_d + bound2det - discprof).^2; 
    
    
    Vsum=Vsum+(disc_Vdet+discprof);
    Vsqr=Vsqr+(disc_Vdet+discprof).^2;   
   
    % Compute tighter bound for different starting states
   
    i=1;
    for start_x=ini_x_bound:dis_x_bound:xmax-dis_x_bound
        term1=0;
        if(2*start_x <= xmax_bound +1)
            for k=0:start_x-1
                if(fix_n~=0)
                    term1=term1+beta^k*max(difprof(start_x-k:start_x+k));
                else
                    term1=term1+beta^k*max(max(difprof(start_x-k:start_x+k)),0);
                end
            end
            for k=start_x:xmax_bound-start_x
                if(fix_n~=0)
                    term1=term1+beta^k*max(difprof(1:start_x+k));
                else
                    term1=term1+beta^k*max(max(difprof(1:start_x+k)),0);
                end
            end
            if(fix_n~=0)
                 term1=term1+beta^(xmax_bound-start_x+1)/(1-beta)*max(difprof(1:xmax_bound));
            else
                 term1=term1+beta^(xmax_bound-start_x+1)/(1-beta)*max(max(difprof(1:xmax_bound)),0);
            end
        else
            for k=0:xmax_bound-start_x
                if(fix_n~=0)
                    term1=term1+beta^k*max(difprof(start_x-k:start_x+k));
                else
                    term1=term1+beta^k*max(max(difprof(start_x-k:start_x+k)),0);
                end
            end
            for k=xmax_bound-start_x+1 : start_x-1
                if(fix_n~=0)
                    term1=term1+beta^k*max(difprof(start_x-k:xmax_bound));
                else
                    term1=term1+beta^k*max(max(difprof(start_x-k:xmax_bound)),0);
                end
            end
            if(fix_n~=0)
                 term1=term1+beta^(start_x)/(1-beta)*max(difprof(1:xmax_bound));
            else
                 term1=term1+beta^(start_x)/(1-beta)*max(max(difprof(1:xmax_bound)),0);
            end
        end
        bound4sum(i) = bound4sum(i) + term1 + bound2det(start_x) - discprof(start_x);
        bound4sqr(i) = bound4sqr(i) + (term1 + bound2det(start_x) - discprof(start_x))^2;
        i=i+1;
    end


    % Tighter bound for model with depreciation
    % First, using estimate of bound calculate number of terms
    % and terms in binomial sum and store them.
    % Alternatively, if memory requirements become too high,
    % could compute the binomial terms each iteration.
    
    if(n==1 || n==10 || n==round(NMIN_bound/2) || n==NMIN_bound)
        kmaxsum=round(log(TOL_precbounds*(max(bound2sum)/n)*(1-beta)/(2*max(proftildes)))/log(beta));
        for k=0:kmaxsum
           for m=0:min(k,xmax_bound-1)
                g(k+1,m+1)= binopdf(m,k,1-delta);
           end
        end
    end
    
    
    % Bound calculations for tighter bound for model with depreciation
    % First, check whether \Delta_y is increasing in y
    if(min(diff(difprofpos))<0)
           error('\Delta_y not increasing in y');
    end
    i=1;
    for start_x=ini_x_bound:dis_x_bound:xmax-dis_x_bound
        term1=0;
        if(max(difprofpos)~=0)
            for k=0:kmaxsum
               binsum=dot(g(k+1,1:min(k,xmax_bound-start_x)+1),...
                    difprofpos(start_x:start_x+min(k,xmax_bound-start_x)));
               if(k>xmax_bound-start_x) % Need to sum extra terms associated to  being at xmax
                  binsum=binsum+(1-binocdf(xmax_bound-start_x,k,1-delta))* difprofpos(xmax_bound); 
               end
               term1=term1+beta^k*binsum;
            end
        end 
        bound5sum(i) = bound5sum(i) + term1 + bound2det(start_x) - discprof(start_x);
        bound5sqr(i) = bound5sqr(i) + (term1 + bound2det(start_x) - discprof(start_x))^2;
        i=i+1;
    end
    
    % Compute industry statistics
    % First compute profits with ignore=1
    % Expected values taken wrt invariant distr. of industry state
    if(ignore==1)
      profindstats=profstate(1:xmax);   msindstats=msstate(1:xmax);  priceindstats=pricesstate(1:xmax);
      prodsurindstats=prodsurstate(1:xmax);  conssurindstats=conssurstate(1:xmax);
    else
      [profindstats,msindstats,priceindstats,prodsurindstats,conssurindstats]=...
         comp_prof(profpar,state,xmax,1);
    end
    
    % Entry/exit related quantities
    if(fix_n==0)  
        numfirms=sum(state);
        entrate=entrants/numfirms;
        entratesum=entratesum+entrate;
        entratesqr=entratesqr+entrate^2;
        numexit=0;
        msexit=0;
        for i=1:xmax
            for j=1:state(i)
                if(rand<=1-prcont(i))
                    numexit=numexit+1; 
                    msexit=msexit+msindstats(i);
                end
            end
        end
        exirate=numexit/numfirms;
        exiratesum=exiratesum+exirate;
        exiratesqr=exiratesqr+exirate^2;
        msexitsum=msexitsum+msexit;
        msexitsqr=msexitsqr+msexit^2;
    end

    % Surplus
    prodsursum=prodsursum+prodsurindstats(1);
    conssursum=conssursum+conssurindstats(1);
    totalsursum=totalsursum+(prodsurindstats(1)+conssurindstats(1));
    prodsursqr=prodsursqr+prodsurindstats(1)^2;
    conssursqr=conssursqr+conssurindstats(1)^2;
    totalsursqr=totalsursqr+(prodsurindstats(1)+conssurindstats(1))^2;
    
    % Concetration ratios
    cumfirms=flipud(cumsum(flipud(state)));
    conc=state.*msindstats;
    cumconc=flipud(cumsum(flipud(conc)));
    c1=0;c4=0;c10=0;c20=0;
    if (cumfirms(xmax)>=1) 
        c1=msindstats(xmax); end
    if (cumfirms(xmax)>=4) 
        c4=msindstats(xmax)*4; end
    if (cumfirms(xmax)>=10) 
        c10=msindstats(xmax)*10; end
    if (cumfirms(xmax)>=20) 
        c20=msindstats(xmax)*20; end
    for i = xmax-1:-1:1
      if (cumfirms(i)>=1 && cumfirms(i+1)<1)
         c1=cumconc(i+1)+msindstats(i)*(1-cumfirms(i+1));   end
      if (cumfirms(i)>=4 && cumfirms(i+1)<4)
         c4=cumconc(i+1)+msindstats(i)*(4-cumfirms(i+1));   end
      if (cumfirms(i)>=10 && cumfirms(i+1)<10)
         c10=cumconc(i+1)+msindstats(i)*(10-cumfirms(i+1)); end
      if (cumfirms(i)>=20 && cumfirms(i+1)<20)
         c20=cumconc(i+1)+msindstats(i)*(20-cumfirms(i+1)); end
    end
    if(cumfirms(1)>=1 && cumfirms(1)<20)  
        c20=1;
        if(cumfirms(1)<10)
            c10=1;
            if(cumfirms(1)<4)
                c4=1;
            end
        end
    end
    c1sum=c1sum+c1;
    c1sqr=c1sqr+c1^2;
    c4sum=c4sum+c4;
    c4sqr=c4sqr+c4^2;
    c10sum=c10sum+c10;
    c10sqr=c10sqr+c10^2;
    c20sum=c20sum+c20;
    c20sqr=c20sqr+c20^2;
   
    if(n>=NMIN_bound)
        [bound1avg,bound1pre,bound1var]=compavgpre(bound1sum,bound1sqr,n);
        [bound2avg,bound2pre,bound2var]=compavgpre(bound2sum,bound2sqr,n);
        [bound4avg,bound4pre,bound4var]=compavgpre(bound4sum,bound4sqr,n);
        [bound5avg,bound5pre,bound5var]=compavgpre(bound5sum,bound5sqr,n);
        [Vavg,Vpre,Vvar]=compavgpre(Vsum,Vsqr,n); 
        %[bound3avg,bound3pre,bound3var]=compavgpre(bound3summin,bound3sqrmin,n);
        boundspre=[bound1pre;bound2pre;bound4pre;bound5pre];
    end
    
    n=n+1;
end  % end while

%difprofsum/n

if(fix_n==0)
    [entrateavg,entratepre,entratevar]=compavgpre(entratesum,entratesqr,n);
    [exirateavg,exiratepre,exiratevar]=compavgpre(exiratesum,exiratesqr,n);
    [msexitavg,msexitpre,msexitvar]=compavgpre(msexitsum,msexitsqr,n);
else
    entrateavg=0;        entratevar=0;
    exirateavg=0;        exiratevar=0;
    msexitavg=0;         msexitvar=0;
end
[prodsuravg,prodsurpre,prodsurvar]=compavgpre(prodsursum,prodsursqr,n);
[conssuravg,conssurpre,conssurvar]=compavgpre(conssursum,conssursqr,n);
[totalsuravg,totalsurpre,totalsurvar]=compavgpre(totalsursum,totalsursqr,n);
[c1avg,c1pre,c1var]=compavgpre(c1sum,c1sqr,n);
[c4avg,c4pre,c4var]=compavgpre(c4sum,c4sqr,n);
[c10avg,c10pre,c10var]=compavgpre(c10sum,c10sqr,n);
[c20avg,c20pre,c20var]=compavgpre(c20sum,c20sqr,n);

bound1por=100*bound1avg./Vavg;
bound2por=100*bound2avg./Vavg;
%i=1;
%bound3por=zeros(size(bound3avg,1),1);
%for start_x=ini_x_bound:dis_x_bound:xmax-dis_x_bound
%    bound3por(i)=100*bound3avg(i)/Vavg(start_x);
%    i=i+1;
%end

i=1;
bound4por=zeros(size(bound4avg,1),1);
for start_x=ini_x_bound:dis_x_bound:xmax-dis_x_bound
    bound4por(i)=100*bound4avg(i)/Vavg(start_x);
    i=i+1;
end

i=1;
bound5por=zeros(size(bound5avg,1),1);
for start_x=ini_x_bound:dis_x_bound:xmax-dis_x_bound
    bound5por(i)=100*bound5avg(i)/Vavg(start_x);
    i=i+1;
end

bound1=[bound1avg*ones(xmax,1),bound1por,bound1pre*ones(xmax,1)];
bound2=[bound2avg,bound2por,bound2pre];
bound4=[bound4avg,bound4por,bound4pre];
bound5=[bound5avg,bound5por,bound5pre];
expV=[Vavg,Vpre];
entryrate=[entrateavg,sqrt(entratevar)];
exitrate=[exirateavg,sqrt(exiratevar)];
msexiters=[msexitavg,sqrt(msexitvar)];
producersur=[prodsuravg,sqrt(prodsurvar)];
consumersur=[conssuravg,sqrt(conssurvar)];
totalsur=[totalsuravg,sqrt(totalsurvar)];
C1=[c1avg,sqrt(c1var)];
C4=[c4avg,sqrt(c4var)];
C10=[c10avg,sqrt(c10var)];
C20=[c20avg,sqrt(c20var)];

 
 
 


