% Main program
% Calls compOE (entry/exit) or compOE_fixn (no entry/no exit) to compute OE
% Calls compbounds to compute bounds


clear variables;

run setup;

outfile=fopen('main.out','w');
format('compact');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compare entry cost with sell-off value and exit if not big enough.

barphi=optstop(constants,transition,control);
if(kappa < beta*barphi)
    error('kappa=%f < beta*barphi=%f. Infinite entry in OE. \n',kappa,beta*barphi);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Print out main parameters of model

fprintf(outfile,'Oblivious Equilibrium Solver\n');
fprintf(outfile,'----------------------------\n\n');
fprintf(outfile,'Profit Function Parameters:\n\n');
fprintf(outfile,'  Market size:     %f\n',m);
fprintf(outfile,'  Consumer Income: %f\n',Y);
fprintf(outfile,'  (theta1,theta2): %f, %f\n',theta1,theta2);
fprintf(outfile,'  psi:             %f\n',psi);
fprintf(outfile,'  Marginal Cost:   %f\n',c);
fprintf(outfile,'  Utility Outside Good:   %f\n',utout);
fprintf(outfile,'\n Transition Probabilities Parameters:\n\n');
fprintf(outfile,'  a:               %f\n',a);
fprintf(outfile,'  delta:           %f\n',delta);
fprintf(outfile,'\n Dynamic Parameters:\n\n');
fprintf(outfile,'  Discount factor: %f\n',beta);
fprintf(outfile,'  Investment Cost: %f\n',d);
fprintf(outfile,'  Entry Cost:      %f\n',kappa);
fprintf(outfile,'  Entry State:     %f\n',xe);
fprintf(outfile,'  Average Sell-off Value:  %f\n',K);
fprintf(outfile,'\n Entry Process:\n');
if(fix_n ~= 0)
    fprintf(outfile,'  Fixed Number of Firms (No entry/No exit): %f\n',fix_n);
else
    if(entryproc==1)
        fprintf(outfile,'  Poisson Entry Process\n');
    else
        fprintf(outfile,'  Constant Number of Entrants\n');
    end
end
fprintf(outfile,'\n Initial Conditions:\n\n');
fprintf(outfile,'  xmax: %f\n',xmax0);
fprintf(outfile,'  Entry Rate: %f, %f\n',lambda10,lambda20);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve for oblivious equilibrium

fprintf(outfile,'\nSolving for OE...\n');

tic;
if(fix_n == 0)
    [iota,rho,lambda,V,xmax,prof,ms,prices,prodsur,conssur,expfreq, ...
    tildes,cond_num] = compOE(outfile,constants,transition,profpar,control,initial);
else
    [iota,rho,lambda,V,xmax,prof,ms,prices,prodsur,conssur,invdistr, ...
    tildes,cond_num] = compOE_fixn(outfile,constants,transition,profpar,control,initial);
end
time=toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output some statistics from OE

if(cond_num<=TOL_condnum)
    fprintf(outfile,'\n WARNING: Heavy-tailed expected industry state. Ill-conditioned matrix to compute stilde\n\n');
end
probsucc = tranprob(transition,iota,xmax);
probsucc = diag([ zeros(xmax-1,1) eye(xmax-1); zeros(1,xmax)]*(probsucc'));
fprintf(outfile,'\n Oblivious Equilibrium Statistics:\n');
fprintf(outfile,'---------------------------------\n\n');
fprintf(outfile,'Oblivious Entry rate: %f\n',lambda);
expnfirms=sum(tildes);
fprintf(outfile,'Expected number of firms: %f\n',expnfirms);

% Calculate C1, C4, C10, C20 at expected state
cumfirms=flipud(cumsum(flipud(tildes)));
conc=tildes.*ms;
cumconc=flipud(cumsum(flipud(conc)));
c1=0;c4=0;c10=0;c20=0;
if (cumfirms(xmax)>=1) 
    c1=ms(xmax); end
if (cumfirms(xmax)>=4) 
    c4=ms(xmax)*4; end
if (cumfirms(xmax)>=10) 
    c10=ms(xmax)*10; end
if (cumfirms(xmax)>=20) 
    c20=ms(xmax)*20; end
for i = xmax-1:-1:1
  if (cumfirms(i)>=1 && cumfirms(i+1)<1)
     c1=cumconc(i+1)+ms(i)*(1-cumfirms(i+1));   end
  if (cumfirms(i)>=4 && cumfirms(i+1)<4)
     c4=cumconc(i+1)+ms(i)*(4-cumfirms(i+1));   end
  if (cumfirms(i)>=10 && cumfirms(i+1)<10)
     c10=cumconc(i+1)+ms(i)*(10-cumfirms(i+1)); end
  if (cumfirms(i)>=20 && cumfirms(i+1)<20)
     c20=cumconc(i+1)+ms(i)*(20-cumfirms(i+1)); end
end
if(cumfirms(1)>=1 && cumfirms(1)<20)  
    c20=1;
    if(cumfirms(1)<10)
        c10=1;
        if(cumfirms(1)<4)
            c4=1;
        end
    end
end
fprintf(outfile,'\n Concentration ratios at expected state:\n\n');
fprintf(outfile,'  C1: %f\n',c1);
fprintf(outfile,'  C4: %f\n',c4);
fprintf(outfile,'  C10: %f\n',c10);
fprintf(outfile,'  C20: %f\n',c20);
if(fix_n==0)
  [profindstats,msindstats,priceindstats,prodsurindstats,conssurindstats]=...
         comp_prof(profpar,tildes,xmax,1);  
  fprintf(outfile,' \n Expected market share of entrants at expected state: %f\n',msindstats(xe)*lambda); 
end

% Expected state, Investment, and Probability of moving up
statenum=(1:1:xmax)';
fprintf(outfile,'\nState     Stilde  Investment  ProbSucc\n');
fprintf(outfile,'%f %f %f %f\n',[statenum tildes iota probsucc]');
fprintf(outfile,'\n Time to compute OE: %f secs.\n',time);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute bounds
fprintf(outfile,'\n Computing bounds...\n');
fclose(outfile);

tic;
ignore=2; % to compute bounds
[bound1,bound2,bound4,bound5,expV,entryrate,exitrate,msexiters,producersur,consumersur,totalsur...
    C1,C4,C10,C20] = compbounds(constants,transition,profpar,control,tildes,lambda,iota,rho,xmax,ignore);
time=toc;


outfile=fopen('main.out','a');
format('compact');

fprintf(outfile,'\n Bound 1 (Loosest bound -- uniform across x)\n');
fprintf(outfile,'---------------------------------------------\n');
fprintf(outfile,'State     Bound   /EV(x,s)    Rel.Prec.\n');
fprintf(outfile,'%f %f %f %f\n',[statenum bound1]');

fprintf(outfile,'\n Bound 2 (Tighter bound -- no restrictions on investment process)\n');
fprintf(outfile,'---------------------------------------------\n');
fprintf(outfile,'State     Bound   /EV(x,s)    Rel.Prec.\n');
fprintf(outfile,'%f %f %f %f\n',[statenum bound2]');

statenum3=(ini_x_bound:dis_x_bound:xmax-dis_x_bound)';
fprintf(outfile,'\n Bound 3 (Tighter bound -- investment process in (-1,0,1)\n');
fprintf(outfile,'---------------------------------------------\n');
fprintf(outfile,'State     Bound   /EV(x,s)    Rel.Prec.\n');
fprintf(outfile,'%f %f %f %f\n',[statenum3 bound4]');

statenum3=(ini_x_bound:dis_x_bound:xmax-dis_x_bound)';
fprintf(outfile,'\n Bound 4 (Tightest bound -- investment process in (-1,0,1) and depreciation considered\n');
fprintf(outfile,'---------------------------------------------\n');
fprintf(outfile,'State     Bound   /EV(x,s)    Rel.Prec.\n');
fprintf(outfile,'%f %f %f %f\n',[statenum3 bound5]');

fprintf(outfile,'\n Time to compute bounds: %f secs.\n',time);


% Print industry statistics
fprintf(outfile,'\n Industry Statistics \n\n');
fprintf(outfile,'Expected value wrt invariant distr. of industry state (standard dev.)\n');
fprintf(outfile,'---------------------------------------------------------------------\n');
fprintf(outfile,'Entry rate (number of entrants/total number of firms): %f (%f)\n',entryrate);
fprintf(outfile,'Exit rate (number of exiters/total number of firms): %f (%f)\n',exitrate);
fprintf(outfile,'Market share exiters: %f (%f)\n',msexiters);
fprintf(outfile,'Producer surplus: %f (%f)\n',producersur);
fprintf(outfile,'Consumer surplus: %f (%f)\n',consumersur);
fprintf(outfile,'Total surplus: %f (%f)\n',totalsur);
fprintf(outfile,'C1: %f (%f)\n',C1);
fprintf(outfile,'C4: %f (%f)\n',C4);
fprintf(outfile,'C10: %f (%f)\n',C10);
fprintf(outfile,'C20: %f (%f)\n',C20);
fclose(outfile);



